import { ChangeDetectionStrategy, Component, inject, signal } from '@angular/core';
import { CommonModule } from '@angular/common';
import { NavbarComponent } from '../../../shared/components/navbar/navbar.component';
import { Router } from '@angular/router';
import { UserService } from '../../../service/user.service';
import { PlayingCardComponent } from './components/playing-card/playing-card.component';
import { DealerHandComponent } from './components/dealer-hand/dealer-hand.component';
import { PlayerHandComponent } from './components/player-hand/player-hand.component';
import { GameControlsComponent } from './components/game-controls/game-controls.component';
import { GameInfoComponent } from './components/game-info/game-info.component';
import { Card, BlackjackGame } from './models/blackjack.model';
import { BlackjackService } from './services/blackjack.service';

@Component({
  selector: 'app-blackjack',
  standalone: true,
  imports: [
    CommonModule,
    NavbarComponent,
    PlayingCardComponent,
    DealerHandComponent,
    PlayerHandComponent,
    GameControlsComponent,
    GameInfoComponent,
  ],
  templateUrl: './blackjack.component.html',
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export default class BlackjackComponent {
  private router = inject(Router);
  private userService = inject(UserService);
  private blackjackService = inject(BlackjackService);

  dealerCards = signal<Card[]>([]);
  playerCards = signal<Card[]>([]);
  currentBet = signal(0);
  balance = signal(0);
  currentGameId = signal<number | undefined>(undefined);
  gameInProgress = signal(false);

  constructor() {
    this.userService.getCurrentUser().subscribe((user) => {
      this.balance.set(user?.balance ?? 0);
    });
  }

  private updateGameState(game: BlackjackGame) {
    console.log('Game state update:', game);
    this.currentGameId.set(game.id);
    this.currentBet.set(game.bet);
    this.gameInProgress.set(game.state === 'IN_PROGRESS');

    this.dealerCards.set(
      game.dealerCards.map((card, index) => ({
        ...card,
        hidden: index === 1 && game.state === 'IN_PROGRESS',
      }))
    );

    this.playerCards.set(
      game.playerCards.map((card) => ({
        ...card,
        hidden: false,
      }))
    );
  }

  onNewGame(bet: number): void {
    this.blackjackService.startGame(bet).subscribe({
      next: (game) => {
        this.updateGameState(game);
      },
      error: (error) => {
        console.error('Failed to start game:', error);
      },
    });
  }

  onHit(): void {
    if (!this.currentGameId()) return;

    this.blackjackService.hit(this.currentGameId()!).subscribe({
      next: (game) => {
        this.updateGameState(game);
      },
      error: (error) => {
        console.error('Failed to hit:', error);
      },
    });
  }

  onStand(): void {
    if (!this.currentGameId()) return;

    this.blackjackService.stand(this.currentGameId()!).subscribe({
      next: (game) => {
        this.updateGameState(game);
      },
      error: (error) => {
        console.error('Failed to stand:', error);
      },
    });
  }

  leaveGame(): void {
    this.router.navigate(['/home']);
  }
}
